;;; -*- lisp -*-
;;; $Id: webview.lisp 495 2003-10-05 16:17:35Z miles $

(in-package :macho)

(defun nice-date (time)
  (format nil "~A.~A.~A" (time-year time) (time-month time) (time-date time)))

(defun nice-time (time)
  (format nil "~A/~A ~D:~2,'0D" (time-month time) (time-date time) (time-hour time) (time-minute time)))

(defun write-archive-index (archive path)
  (msg "writing archive index to ~A~%" path)
  (ensure-directories-exist path)
  (with-slots (name latest-message-date messages year-table) archive
    (let* ((year (this-year))
           (month (this-month))
           (years (sort (loop for k being each hash-key of year-table collect k) #'>))
           (msgs (loop for year in years
                       for msgs = (gethash year year-table)
                       collect `(:year ,(s+ year)
                                 :months ,(loop for i from 0 to 11
                                                for m = (svref msgs i)
                                                for count = (length m)
                                                collect `(:link ,(s+ year "-" (1+ i) "/index.html")
                                                          :link-text ,(s+ count)
                                                          :count ,(if (> count 0) count nil)))))))
      (with-open-file (index path :direction :output)
        (let ((vars `(:name ,name
                      :stylesheet "../../macho.css"
                      :last-message-date ,(nice-date latest-message-date)
                      :total-messages ,(s+ (length messages))
                      :years ,(loop for year in years collect (s+ year))
                      :msgs ,msgs
                      :months ,(loop for i from 1 to 12 
                                     collect `(:value ,(format nil "~3,3/net.telent.date:monthname/" i))))))
          (html-template:fill-and-print-template #p"templates/archive-index.html"
                                                 vars
                                                 :stream index))))))

(defun message-path (message)
  (with-slots (time serial) message
    (s+ "../" (time-year time) "-" (time-month time) "/" serial ".html")))
  
(defun month-index-entry (msg depth num)
  (with-slots (serial subject author time) msg
    `(:href ,(message-path msg)
      :number ,(s+ num)
      :depth ,(s+ (* 2 depth))
      :author ,(s+ author)
      :subject ,(s+ subject)
      :date ,(nice-time time))))

(defun build-month-index-entries (archive year month)
  (with-slots (year-table) archive
    (loop for msg in (svref (gethash year year-table) month)
          for i upfrom 1
          collect (month-index-entry msg 0 i))))

(defun build-month-thread-index-entries (archive year month)
  (with-slots (year-table) archive
    (let ((current 0)
          (msgs nil))
      (labels ((build-msg (msg &optional (depth 0))
                 (let ((msg-vals (month-index-entry msg depth current)))
                   (push msg-vals msgs)
                   (if (message-children msg)
                       (loop for child in (message-children msg)
                             do (progn
                                  (incf current)
                                  (build-msg child (1+ depth))))))))
        (loop for msg in (svref (gethash year year-table) month)
              if (not (and (message-parent msg)
                           (= (time-month (message-time (message-parent msg)))
                              (1+ month))))
              do (progn
                   (incf current)
                   (build-msg msg))))
      (nreverse msgs))))

(defun write-month-index (archive year month msgs path)
  (msg "writing month index for ~A / ~A to ~A~%" year (1+ month) path)
  (ensure-directories-exist path)
  (with-open-file (index path :direction :output)
    (with-slots (name messages year-table) archive
      (let* ((title (format nil "~A mail archive - ~/net.telent.date:monthname/ ~A" name (1+ month) year))
             (vars `(:archive-name ,name
                     :title ,title
                     :stylesheet "../../../macho.css"
                     :year ,(s+ year)
                     :month ,(s+ (1+ month))
                     :msgs ,msgs)))
        (html-template:fill-and-print-template #p"templates/month-index.html" vars :stream index)))))

(defun write-message (path msg &optional prev next)
  (ensure-directories-exist path)
  (with-open-file (stream path :direction :output)
    (with-slots (archive author children serial subject time id in-reply) msg
      (msg "writing msg ~S ~S ~S ~S~%" serial author subject id)
      (let ((vars `(:title ,(format nil "~A mail archive - : ~A (~A)" (archive-name archive) subject author)
                    :stylesheet "../../../macho.css"
                    :name ,(archive-name archive)
                    :date ,(s+ (time-year time) "-" (time-month time) "-" (time-date time))
                    :author ,author
                    :subject ,subject
                    :message-date ,(format nil "~A-~A-~A ~D:~2,'0D" 
                                           (time-year time)
                                           (time-month time)
                                           (time-date time)
                                           (time-hour time)
                                           (time-minute time))
                    :prev ,(if prev (message-path prev))
                    :next ,(if next (message-path next))
                    :thread-next ,(if children
                                      (s+ (message-serial (first children)) ".html"))
                    :body ,(obscure-email-addresses (message-body msg)))))
        (html-template:fill-and-print-template #p"templates/message.html" vars :stream stream)))))

(defun write-archive (archive path)
  (msg "building web pages for ~S~%" (archive-name archive))
  (with-slots (name messages year-table dirty-table) archive
    (write-archive-index archive (join-path path "index.html"))
    (loop for year being each hash-key of year-table
          using (hash-value msgs)
          do (if (gethash year dirty-table)
                 (progn
                   (dotimes (month 12)
                     (let ((month-dir (join-path path (s+ year "-" (1+ month))))
                           (month-dirty (svref (gethash year dirty-table) month)))
                       (if month-dirty
                           (progn
                             (loop for name in (list "author" "date" "subject")
                                   for pred in (list #'string< #'time< #'string<)
                                   for key in (list #'message-author #'message-time #'message-subject)
                                   do (loop for suffix in (list "ascending" "descending")
                                            for pred2 in (list pred #'(lambda (a b) (not (funcall pred a b))))
                                            do (progn
                                                 (setf (svref msgs month)
                                                       (stable-sort (svref msgs month) pred2 :key key))
                                                 (write-month-index archive
                                                                    year
                                                                    month
                                                                    (build-month-index-entries archive year month)
                                                                    (join-path month-dir (s+ name "-" suffix ".html"))))))
                             (setf (svref msgs month)
                                   (stable-sort (svref msgs month) #'time< :key #'message-time))
                             (write-month-index archive
                                                year
                                                month
                                                (build-month-thread-index-entries archive year month)
                                                (join-path month-dir "thread-ascending.html"))
                             (setf (svref msgs month)
                                   (stable-sort (svref msgs month) #'time> :key #'message-time))
                             (write-month-index archive
                                                year
                                                month
                                                (build-month-thread-index-entries archive year month)
                                                (join-path month-dir "thread-descending.html"))
                             (let ((index-path (join-path month-dir "index.html")))
                               (if (probe-file index-path)
                                   (sb-posix:unlink index-path))
                               (sb-posix:symlink "thread-descending.html" index-path))
                             (if (and (= (1+ month) (this-month)) (= year (this-year)))
                                 (let ((latest-path (join-path path "latest")))
                                   (if (probe-file latest-path)
                                       (sb-posix:unlink (join-path path "latest")))
                                   (sb-posix:symlink (s+ year "-" (1+ month)) latest-path)))
                             (loop with msgs = (svref msgs month)
                                   with last = (length messages)
                                   for msg in msgs
                                   for serial = (message-serial msg)
                                   do (if (message-dirty msg)
                                          (write-message (join-path month-dir (s+ serial ".html"))
                                                         msg
                                                         (if (= serial 1) nil (aref messages (- serial 2)))
                                                         (if (= serial last) nil (aref messages serial))))))))))))))

